<?php

namespace Drupal\nostr_content_nip23\Services;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * NostrFormAlter service class.
 */
class NostrFormAlter implements NostrFormAlterInterface {

  use StringTranslationTrait;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected MessengerInterface $messenger;

  /**
   * Nostr storage service.
   *
   * @var \Drupal\nostr_content_nip23\Services\NostrStorageInterface
   */
  protected NostrStorageInterface $nostrStorage;

  /**
   * Nostr publish service.
   *
   * @var \Drupal\nostr_content_nip23\Services\NostrPublishInterface
   */
  protected NostrPublishInterface $nostrPublish;

  /**
   * NostrFormAlter constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Drupal\nostr_content_nip23\Services\NostrStorageInterface $nostr_storage
   *   The nostr storage.
   * @param \Drupal\nostr_content_nip23\Services\NostrPublishInterface $nostr_publish
   *   The nostr publish message.
   */
  public function __construct(AccountInterface $current_user, MessengerInterface $messenger, NostrStorageInterface $nostr_storage, NostrPublishInterface $nostr_publish) {
    $this->currentUser = $current_user;
    $this->nostrStorage = $nostr_storage;
    $this->messenger = $messenger;
    $this->nostrPublish = $nostr_publish;
  }

  /**
   * {@inheritdoc}
   */
  public function addNostrFormElement(&$form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $form_state->getFormObject()->getEntity();
    if ($entity && $this->currentUser->hasPermission('publish to nostr network')) {

      $can_publish = FALSE;
      $can_republish = FALSE;
      $event_id = NULL;
      if ($entity->isNew() || !($event_id = $this->nostrStorage->getEventId($entity->id(), $entity->getEntityTypeId()))) {
        $can_publish = TRUE;
      } else {
        $can_republish = TRUE;
      }

      // @todo if we have a $event_id, generate a nevent bech32-encoded identifier
      // see https://github.com/nostr-protocol/nips/blob/master/19.md

      $form['nostr_wrapper'] = [
        '#type' => 'details',
        '#title' => $this->t('Nostr'),
        '#group' => 'advanced',
      ];

      if ($can_publish === TRUE) {

        $form['nostr_wrapper']['nostr_publish'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Publish to the Nostr network'),
        ];

        foreach (array_keys($form['actions']) as $action) {
          if ($action === 'submit' && isset($form['actions'][$action]['#type']) && $form['actions'][$action]['#type'] === 'submit') {
            $form['actions'][$action]['#submit'][] = [
              $this,
              'submitNostrFormElement',
            ];
          }
        }
      }
      else if ($can_republish === TRUE) {
        $form['nostr_wrapper']['sent'] = [
          '#markup' => '<p>' . $this->t('This post is signed by [npub_here] and has been published to the Nostr network.<br /><br />View: <a href="https://njump.me/@id" target="_blank">@id</a>', ['@id' => $event_id]) . '</p>',
        ];

        $form['nostr_wrapper']['nostr_publish'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Re-publish to the Nostr network'),
        ];

        foreach (array_keys($form['actions']) as $action) {
          if ($action === 'submit' && isset($form['actions'][$action]['#type']) && $form['actions'][$action]['#type'] === 'submit') {
            $form['actions'][$action]['#submit'][] = [
              $this,
              'submitNostrFormElement',
            ];
          }
        }
      }
      else {
        $form['nostr_wrapper']['sent'] = [
          '#markup' => '<p>' . $this->t('This post is sign by [npub_here] and has been published to the Nostr network.<br /><br />View: <a href="https://njump.me/@id" target="_blank">@id</a>', ['@id' => $event_id]) . '</p>',
        ];
      }
    }
  }

  /**
   * Submit function to send a post to the Nostr network.
   *
   * @param array $form
   *   Structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Current form state.
   */
  public function submitNostrFormElement(array $form, FormStateInterface $form_state) {
    if ($form_state->getValue('nostr_publish')) {

      // ----------------------------------------------------------
      // Validate file and contents.
      $public_key_file = Settings::get('nostr_public_key_file');
      if (!file_exists($public_key_file)) {
        $this->messenger->addWarning($this->t('The file which contains your Nostr public key has not been found.'));
        return;
      }

      $private_key_file = Settings::get('nostr_private_key_file');
      if (!file_exists($private_key_file)) {
        $this->messenger->addWarning($this->t('The file which contains your Nostr private key has not been found.'));
        return;
      }

      $public_key = trim(file_get_contents($public_key_file));
      if (empty($public_key)) {
        $this->messenger->addWarning($this->t('The Nostr public key file is empty.'));
        return;
      }

      $private_key = trim(file_get_contents($private_key_file));
      if (empty($private_key)) {
        $this->messenger->addWarning($this->t('The Nostr private key file is empty.'));
        return;
      }

      $relay = Settings::get('nostr_relay');
      if (empty($relay)) {
        $this->messenger->addWarning($this->t('The Nostr relay is empty.'));
        return;
      }

      // ----------------------------------------------------------
      // All good, let's publish!
      /** @var \Drupal\Core\Entity\EntityInterface $entity */
      $entity = $form_state->getFormObject()->getEntity();
      $this->nostrPublish->sendMessage($entity, $public_key, $private_key, $relay, TRUE);
    }

  }

}
